#!/usr/bin/env node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { colarg } from "colarg";
import * as fs from "fs";
import * as path from "path";
var cli = new colarg(process.argv.slice(2));
cli.addOption({
    name: "locale",
    alias: "l",
    description: "A specific locale to use when generating the output.",
    required: false,
    type: "string"
});
cli.addOption({
    name: "map",
    alias: "m",
    description: "A JSON file containing a map of all locales available, this will cause iteration over all locales and output into corresponding folders/files.",
    required: true,
    type: "string"
});
cli.addOption({
    name: "output",
    alias: "o",
    description: "A path to output the converted file to.",
    required: false,
    type: "string"
});
cli.addOption({
    name: "output-type",
    alias: "t",
    description: "The method to output converted files, either 'file' or 'folder', 'file' will cause the output to be written as a file into the current folder, 'folder' will generate folders with the locales as name and output the original file name into that.",
    required: false,
    defaults: "folder",
    type: "string"
});
cli.addOption({
    name: "i18n-prefixed",
    alias: "i",
    description: "Whether to prefix injected variables with 'i18n'.",
    required: false,
    defaults: true,
    type: "boolean"
});
cli.enableHelp();
var args = cli.getArgs();
// First of all, we need to get the input file, this will be the first default argument.
if (!args["default"] || args["default"].length == 0) {
    throw new Error("Missing required argument for input file, make sure it is passed as anonymous argument.");
}
var inputFile = args["default"][0];
// Check if the given path is absolute or relative, if it's relative we will have to pad it with the current working directory.
var inputContent;
if (fs.existsSync(inputFile)) {
    // The path is absolute, read the file.
    inputContent = fs.readFileSync(inputFile).toString();
}
else {
    // Pad the path with the current working directory
    inputContent = fs.readFileSync(process.cwd() + '/' + inputFile).toString();
}
(function () {
    return __awaiter(this, void 0, void 0, function () {
        var locales, localeName;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!args.map) return [3 /*break*/, 4];
                    locales = void 0;
                    if (!(path.extname(args.map) == ".js")) return [3 /*break*/, 2];
                    return [4 /*yield*/, import(path.join(process.cwd(), args.map))];
                case 1:
                    locales = (_a.sent())["default"];
                    return [3 /*break*/, 3];
                case 2:
                    if (path.extname(args.map) == ".json") {
                        locales = JSON.parse(fs.readFileSync(getAbsolute(args.map)).toString());
                    }
                    _a.label = 3;
                case 3:
                    // Check if only one locale should be used as output.
                    if (args.locale) {
                        translate(locales, args.locale);
                    }
                    else {
                        // Otherwise we will loop through the locale map.
                        for (localeName in locales) {
                            translate(locales, localeName);
                        }
                    }
                    _a.label = 4;
                case 4: return [2 /*return*/];
            }
        });
    });
})();
function translate(locales, localeName) {
    var translations = locales[localeName];
    // We now have a list of translations, a list of key-value pairs with translation names as keys and their corresponding translation values as values.
    var translatedFile = inputContent;
    for (var wordName in translations) {
        var translation = translations[wordName];
        // Check if we need to prefix the name with 'i18n'
        if (args["i18n-prefixed"]) {
            wordName = "i18n_" + wordName;
        }
        translatedFile = translatedFile.replace(new RegExp(wordName, 'g'), "\"".concat(translation.replace(/"/g, '\\"').replace(/\n/g, '\\n'), "\""));
    }
    // The file has been translated, let's write it to the output.
    // If the output-type is 'folder', we need to create multiple subfolders where we output the file name.
    var fileName = path.basename(inputFile);
    // The output argument will be used as the root folder for the output.
    var rootFolder = getAbsolute(args.output);
    if (args["output-type"] == "folder") {
        // Maybe create the output folder first.
        var outputFolder = path.join(rootFolder, localeName);
        if (!fs.existsSync(outputFolder)) {
            fs.mkdirSync(outputFolder, { recursive: true });
        }
        fs.writeFileSync(path.join(outputFolder, fileName), translatedFile);
    }
    else {
        fs.writeFileSync(path.join(rootFolder, localeName + "_" + fileName), translatedFile);
    }
}
function getAbsolute(p) {
    if (fs.existsSync(p)) {
        return p;
    }
    else {
        return path.join(process.cwd(), p);
    }
}
